@@ -3,7 +3,7 @@ package Calendar::Indonesia::Holiday;
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
 our $DATE = '2021-01-14'; # DATE
 our $DIST = 'Calendar-Indonesia-Holiday'; # DIST
-our $VERSION = '0.333'; # VERSION
+our $VERSION = '0.334'; # VERSION
 
 use 5.010001;
 use strict;
@@ -184,8 +184,8 @@ sub _h_eidulf {
         ($opts->{hyear} ? " $opts->{hyear}H":"");
     my $eng_name0     = "Eid Ul-Fitr".
         ($opts->{hyear} ? " $opts->{hyear}H":"");
-    $r->{ind_name}    = $ind_name0.($opts->{day} ? ", Hari $opts->{day}":"");
-    $r->{eng_name}    = $eng_name0.($opts->{day} ? ", Day $opts->{day}":"");
+    $r->{ind_name}    = $ind_name0.($opts->{day} ? " (Hari $opts->{day})":"");
+    $r->{eng_name}    = $eng_name0.($opts->{day} ? " (Day $opts->{day})":"");
     _add_original_date($r, $opts);
     $r->{ind_aliases} = ["Lebaran"];
     $r->{eng_aliases} = [];
@@ -296,8 +296,230 @@ sub _expand_dm {
     return (day => $1+0, month => $2+0);
 }
 
+sub _uniquify_holidays {
+    my @holidays = @_;
+
+    my %seen; # key=mm-dd (e.g. 12-25), val=[hol1, hol2, ...]
+    for my $h (@holidays) {
+        my $k = sprintf "%02d-%02d", $h->{month}, $h->{day};
+        $seen{$k} //= [];
+        push @{ $seen{$k} }, $h;
+    }
+
+    for my $k (keys %seen) {
+        if (@{ $seen{$k} } == 1) {
+            $seen{$k} = $seen{$k}[0];
+        } else {
+            my $h_mult = {
+                multiple => 1,
+                ind_name => join(", ", map {$_->{ind_name}} @{ $seen{$k} }),
+                eng_name => join(", ", map {$_->{eng_name}} @{ $seen{$k} }),
+                holidays => $seen{$k},
+            };
+            # join all the tags
+            my @tags;
+            for my $h (@{ $seen{$k} }) {
+                next unless $h->{tags};
+                for my $t (@{ $h->{tags} }) {
+                    push @tags, $t unless grep { $_ eq $t } @tags;
+                }
+            }
+            $h_mult->{tags} = \@tags;
+            # join all the properties
+          PROP:
+            for my $prop (keys %{ $seen{$k}[0] }) {
+                next if exists $h_mult->{$prop};
+                my %vals;
+                for my $h (@{ $seen{$k} }) {
+                    next PROP unless defined $h->{$prop};
+                    $vals{ $h->{$prop} }++;
+                }
+                next if keys(%vals) > 1;
+                $h_mult->{$prop} = $seen{$k}[0]{$prop};
+            }
+            $seen{$k} = $h_mult;
+        }
+    }
+
+    map { $seen{$_} } sort keys %seen;
+}
+
 my %year_holidays;
 
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1990
+{
+    $year_holidays{1990} = [
+        _h_isramiraj ({_expand_dm("23-02")}, {hyear=>1410}),
+        _h_nyepi     ({_expand_dm("27-03")}, {hyear=>1912}),
+        _h_goodfri   ({_expand_dm("13-04")}),
+        _h_eidulf    ({_expand_dm("26-04")}, {hyear=>1410, day=>1}),
+        _h_eidulf    ({_expand_dm("27-04")}, {hyear=>1410, day=>2}),
+        _h_vesakha   ({_expand_dm("10-05")}, {hyear=>2534}),
+        _h_ascension ({_expand_dm("24-05")}),
+        _h_eidula    ({_expand_dm("03-07")}, {hyear=>1410}),
+        _h_hijra     ({_expand_dm("23-07")}, {hyear=>1411}),
+        _h_mawlid    ({_expand_dm("01-10")}, {hyear=>1411}),
+    ];
+}
+
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1991
+{
+    $year_holidays{1991} = [
+        _h_isramiraj ({_expand_dm("12-02")}, {hyear=>1411}),
+        _h_nyepi     ({_expand_dm("17-03")}, {hyear=>1913}),
+        _h_goodfri   ({_expand_dm("29-03")}),
+        _h_eidulf    ({_expand_dm("16-04")}, {hyear=>1411, day=>1}),
+        _h_eidulf    ({_expand_dm("17-04")}, {hyear=>1411, day=>2}),
+        _h_ascension ({_expand_dm("09-05")}),
+        _h_vesakha   ({_expand_dm("28-05")}, {hyear=>2535}),
+        _h_eidula    ({_expand_dm("23-06")}, {hyear=>1411}),
+        _h_hijra     ({_expand_dm("13-07")}, {hyear=>1412}),
+        _h_mawlid    ({_expand_dm("21-09")}, {hyear=>1412}),
+    ];
+}
+
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1992
+{
+    $year_holidays{1992} = [
+        _h_isramiraj ({_expand_dm("01-02")}, {hyear=>1412}),
+        _h_nyepi     ({_expand_dm("05-03")}, {hyear=>1914}),
+        _h_eidulf    ({_expand_dm("05-04")}, {hyear=>1412, day=>1}),
+        _h_eidulf    ({_expand_dm("06-04")}, {hyear=>1412, day=>2}),
+        _h_goodfri   ({_expand_dm("17-04")}),
+        _h_vesakha   ({_expand_dm("16-05")}, {hyear=>2536}),
+        _h_ascension ({_expand_dm("28-05")}),
+        _h_lelection ({_expand_dm("09-06")}, {}),
+        _h_eidula    ({_expand_dm("11-06")}, {hyear=>1412}),
+        _h_hijra     ({_expand_dm("02-07")}, {hyear=>1413}),
+        _h_mawlid    ({_expand_dm("09-09")}, {hyear=>1413}),
+    ];
+}
+
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1993
+{
+    $year_holidays{1993} = [
+        _h_isramiraj ({_expand_dm("20-01")}, {hyear=>1413}),
+        _h_nyepi     ({_expand_dm("24-03")}, {hyear=>1915}),
+        _h_eidulf    ({_expand_dm("25-03")}, {hyear=>1413, day=>1}),
+        _h_eidulf    ({_expand_dm("26-03")}, {hyear=>1413, day=>2}),
+        _h_goodfri   ({_expand_dm("09-04")}),
+        _h_vesakha   ({_expand_dm("06-05")}, {hyear=>2537}),
+        _h_ascension ({_expand_dm("20-05")}),
+        _h_eidula    ({_expand_dm("01-06")}, {hyear=>1413}),
+        _h_hijra     ({_expand_dm("21-06")}, {hyear=>1414}),
+        _h_mawlid    ({_expand_dm("30-08")}, {hyear=>1414}),
+    ];
+}
+
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1994
+{
+    $year_holidays{1994} = [
+        _h_isramiraj ({_expand_dm("10-01")}, {hyear=>1414}),
+        _h_eidulf    ({_expand_dm("14-03")}, {hyear=>1414, day=>1}),
+        _h_eidulf    ({_expand_dm("15-03")}, {hyear=>1414, day=>2}),
+        _h_goodfri   ({_expand_dm("01-04")}),
+        _h_nyepi     ({_expand_dm("12-04")}, {hyear=>1916}),
+        _h_ascension ({_expand_dm("12-05")}),
+        _h_eidula    ({_expand_dm("21-05")}, {hyear=>1414}),
+        _h_vesakha   ({_expand_dm("25-05")}, {hyear=>2538}),
+        _h_hijra     ({_expand_dm("11-06")}, {hyear=>1415}),
+        _h_mawlid    ({_expand_dm("20-08")}, {hyear=>1415}),
+        _h_isramiraj ({_expand_dm("30-12")}, {hyear=>1415}),
+    ];
+}
+
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1995
+{
+    $year_holidays{1995} = [
+        _h_eidulf    ({_expand_dm("03-03")}, {hyear=>1415, day=>1}),
+        _h_eidulf    ({_expand_dm("04-03")}, {hyear=>1415, day=>2}),
+        _h_nyepi     ({_expand_dm("01-04")}, {hyear=>1917}),
+        _h_goodfri   ({_expand_dm("14-04")}),
+        _h_eidula    ({_expand_dm("10-05")}, {hyear=>1415}),
+        _h_vesakha   ({_expand_dm("15-05")}, {hyear=>2539}),
+        _h_ascension ({_expand_dm("25-05")}),
+        _h_hijra     ({_expand_dm("31-05")}, {hyear=>1416}),
+        _h_mawlid    ({_expand_dm("09-08")}, {hyear=>1416}),
+        _h_isramiraj ({_expand_dm("20-12")}, {hyear=>1416}),
+    ];
+}
+
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1996
+{
+    $year_holidays{1996} = [
+        _h_eidulf    ({_expand_dm("20-02")}, {hyear=>1416, day=>1}),
+        _h_eidulf    ({_expand_dm("21-02")}, {hyear=>1416, day=>2}),
+        _h_nyepi     ({_expand_dm("21-03")}, {hyear=>1918}),
+        _h_goodfri   ({_expand_dm("05-04")}),
+        _h_eidula    ({_expand_dm("28-04")}, {hyear=>1416}),
+        _h_ascension ({_expand_dm("16-05")}),
+        _h_hijra     ({_expand_dm("19-05")}, {hyear=>1417}),
+        _h_vesakha   ({_expand_dm("02-06")}, {hyear=>2540}),
+        _h_mawlid    ({_expand_dm("28-07")}, {hyear=>1417}),
+        _h_isramiraj ({_expand_dm("08-12")}, {hyear=>1417}),
+    ];
+}
+
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1997
+{
+    $year_holidays{1997} = [
+        _h_eidulf    ({_expand_dm("09-02")}, {hyear=>1417, day=>1}),
+        _h_eidulf    ({_expand_dm("10-02")}, {hyear=>1417, day=>2}),
+        _h_goodfri   ({_expand_dm("28-03")}),
+        _h_nyepi     ({_expand_dm("09-04")}, {hyear=>1919}),
+        _h_eidula    ({_expand_dm("18-04")}, {hyear=>1417}),
+        _h_hijra     ({_expand_dm("08-05")}, {hyear=>1418}), # coincide
+        _h_ascension ({_expand_dm("08-05")}),                # coincide
+        _h_vesakha   ({_expand_dm("22-05")}, {hyear=>2541}),
+        _h_lelection ({_expand_dm("29-05")}, {}),
+        _h_mawlid    ({_expand_dm("17-07")}, {hyear=>1418}),
+        _h_isramiraj ({_expand_dm("28-11")}, {hyear=>1418}),
+    ];
+}
+
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1998
+{
+    $year_holidays{1998} = [
+        _h_eidulf    ({_expand_dm("30-01")}, {hyear=>1418, day=>1}),
+        _h_eidulf    ({_expand_dm("31-01")}, {hyear=>1418, day=>2}),
+        _h_nyepi     ({_expand_dm("29-03")}, {hyear=>1920}),
+        _h_eidula    ({_expand_dm("07-04")}, {hyear=>1418}),
+        _h_goodfri   ({_expand_dm("10-04")}),
+        _h_hijra     ({_expand_dm("28-04")}, {hyear=>1419}),
+        _h_vesakha   ({_expand_dm("11-05")}, {hyear=>2542}),
+        _h_ascension ({_expand_dm("21-05")}),
+        _h_mawlid    ({_expand_dm("06-07")}, {hyear=>1419}),
+        _h_isramiraj ({_expand_dm("17-11")}, {hyear=>1419}),
+    ];
+}
+
+# decreed ?
+# source: https://id.wikipedia.org/wiki/1999
+{
+    $year_holidays{1999} = [
+        _h_eidulf    ({_expand_dm("19-01")}, {hyear=>1419, day=>1}),
+        _h_eidulf    ({_expand_dm("20-01")}, {hyear=>1419, day=>2}),
+        _h_nyepi     ({_expand_dm("18-03")}, {hyear=>1921}),
+        _h_eidula    ({_expand_dm("28-03")}, {hyear=>1419}),
+        _h_goodfri   ({_expand_dm("02-04")}),
+        _h_hijra     ({_expand_dm("17-04")}, {hyear=>1420}),
+        _h_ascension ({_expand_dm("13-05")}),
+        _h_vesakha   ({_expand_dm("30-05")}, {hyear=>2543}),
+        _h_lelection ({_expand_dm("07-06")}, {}),
+        _h_mawlid    ({_expand_dm("26-06")}, {hyear=>1420}),
+        _h_isramiraj ({_expand_dm("06-11")}, {hyear=>1420}),
+    ];
+}
+
 # decreed ?
 # source: https://id.wikipedia.org/wiki/2000
 {
@@ -392,6 +614,7 @@ my %year_holidays;
         _h_eidula    ({_expand_dm("02-02")}, {hyear=>1424, original_date=>'2004-02-01'}),
         _h_hijra     ({_expand_dm("23-02")}, {hyear=>1425, original_date=>'2004-02-01'}),
         _h_nyepi     ({_expand_dm("22-03")}, {hyear=>1926}),
+        _h_lelection ({_expand_dm("05-04")}, {}),
         _h_goodfri   ({_expand_dm("09-04")}),
         _h_mawlid    ({_expand_dm("03-05")}, {original_date=>'2004-05-02'}),
         _h_ascension ({_expand_dm("20-05")}),
@@ -822,6 +1045,7 @@ my %year_holidays;
         _h_chnewyear ({_expand_dm("05-02")}, {hyear=>2570}),
         _h_nyepi     ({_expand_dm("07-03")}, {hyear=>1941}),
         _h_isramiraj ({_expand_dm("03-04")}, {hyear=>1440}),
+        _h_lelection ({_expand_dm("17-04")}),
         _h_goodfri   ({_expand_dm("19-04")}),
         # - labor day
         _h_vesakha   ({_expand_dm("19-05")}, {hyear=>2563}),
@@ -994,7 +1218,7 @@ for my $year ($min_year .. $max_year) {
         $h->{dow}  = $dt->day_of_week;
     }
 
-    push @holidays, (sort {$a->{date} cmp $b->{date}} @hf, @hy);
+    push @holidays, _uniquify_holidays(@hf, @hy);
 }
 
 my $res = gen_read_table_func(
@@ -1354,7 +1578,7 @@ Calendar::Indonesia::Holiday - List Indonesian public holidays
 
 =head1 VERSION
 
-This document describes version 0.333 of Calendar::Indonesia::Holiday (from Perl distribution Calendar-Indonesia-Holiday), released on 2021-01-14.
+This document describes version 0.334 of Calendar::Indonesia::Holiday (from Perl distribution Calendar-Indonesia-Holiday), released on 2021-01-14.
 
 =head1 SYNOPSIS
 
@@ -1446,7 +1670,7 @@ days*. If work_saturdays is set to true, Saturdays are also counted as working
 days. If observe_joint_leaves is set to false, joint leave days are also counted
 as working days.
 
-Contains data from years 2000 to 2021 (joint leave days until
+Contains data from years 1990 to 2021 (joint leave days until
 2021).
 
 This function is not exported by default, but exportable.
@@ -1506,7 +1730,7 @@ days*. If work_saturdays is set to true, Saturdays are also counted as working
 days. If observe_joint_leaves is set to false, joint leave days are also counted
 as working days.
 
-Contains data from years 2000 to 2021 (joint leave days until
+Contains data from years 1990 to 2021 (joint leave days until
 2021).
 
 This function is not exported by default, but exportable.
@@ -1618,7 +1842,7 @@ List Indonesian holidays in calendar.
 
 List holidays and joint leave days ("cuti bersama").
 
-Contains data from years 2000 to 2021 (joint leave days until
+Contains data from years 1990 to 2021 (joint leave days until
 2021).
 
 This function is not exported by default, but exportable.
@@ -2113,7 +2337,32 @@ officially decreed by the government.
 
 [5] https://kalenderindonesia.com/libur/masehi/1991
 
-=head2 Holidays before 2002?
+=head2 What happens when multiple religious/holidays coincide on a single calendar day?
+
+For example, in 1997, both Hijra and Ascension Day fall on May 8th. When this
+happens, C<ind_name> and C<eng_name> will contain all the names of the holidays
+separated by comma, respectively:
+
+ Tahun Baru Hijriah, Kenaikan Isa Al-Masih
+ Hijra, Ascension Day
+
+All the properties that have the same value will be set in the merged holiday
+data:
+
+ is_holiday => 1,
+ is_joint_leave => 1,
+
+The C<multiple> property will also be set to true:
+
+ multiple => 1,
+
+All the tags will be merged:
+
+ tags => ['religious', 'religion=christianity', 'calendar=lunar']
+
+You can get each holiday's data in the C<holidays> key.
+
+=head2 Data for older holidays?
 
 Will be provided if there is demand and data source.
 
